<?php

/**
 * This plugin displays an icon showing the status 
 * of dkim verification of the message 
 *
 * @version 0.4.1
 * @author Julien vehent 
 * tested with 
 * - dkimproxy 1.3 beta 1
 * - Amavis 2.6.1
 * - both using Mail::DKIM 0.32
 * @mail julien@linuxwall.info
 * 
 * original plugin from Vladimir Mach - wladik@gmail.com
 * http://www.wladik.net
 *
 * Changelog:
 *	20100202 - fix for amavis and add cz translation
 *	20100201 - add control of header.i and header.from to detect third party signature, change icons
 *	20100115 - add 'no information' status with image using x-dkim-authentication-results
 *	20090920 - fixed space in matching status (thanks Pim Pronk for suggestion)
 */
class dkimstatus extends rcube_plugin
{
	public $task = 'mail';
	function init()
	{
		$rcmail = rcmail::get_instance();
		if ($rcmail->action == 'show' || $rcmail->action == 'preview') {
			$this->add_hook('imap_init', array($this, 'imap_init'));
			$this->add_hook('message_headers_output', array($this, 'message_headers'));
		} else if ($rcmail->action == '') {
				// with enabled_caching we're fetching additional headers before show/preview
				$this->add_hook('imap_init', array($this, 'imap_init'));
			}
	}
	
	function imap_init($p)
	{
		$rcmail = rcmail::get_instance();
		$p['fetch_headers'] = trim($p['fetch_headers'].' ' . strtoupper('Authentication-Results').' '. strtoupper('X-DKIM-Authentication-Results'));
		return $p;
	}
	
	function message_headers($p)
	{
		$this->add_texts('localization');

		/* First, if dkimproxy did not find a signature, stop here
		*/
		if($p['headers']->others['x-dkim-authentication-results'] || $p['headers']->others['authentication-results']){

			$results = $p['headers']->others['x-dkim-authentication-results'];

			if(preg_match("/none/", $results)) 
			{
				$p['output']['from']['value'] = '<img src="plugins/dkimstatus/images/nosiginfo.png" alt="'.$this->gettext('nosignature').'" title="'.$this->gettext('nosignature').'" /> ' . $p['output']['from']['value'];
			}
			else {
				/* Second, check the authentication-results header
				*/
				if($p['headers']->others['authentication-results']) {

					$results = $p['headers']->others['authentication-results'];

					if(preg_match("/dkim=([a-zA-Z0-9]*)/", $results, $m)) {
						$status = ($m[1]);
					}

					if(preg_match("/domainkeys=([a-zA-Z0-9]*)/", $results, $m)) {
						$status = ($m[1]);
					}


					if($status == 'pass') {


						/* Verify if its an author's domain signature or a third party
						*/


						if(preg_match("/[@][a-zA-Z0-9]+([.][a-zA-Z0-9]+)?\.[a-zA-Z]{2,4}/", $p['headers']->from, $m))
						{
							$authordomain = $m[0];
							if(preg_match("/header\.i=(([a-zA-Z0-9]+[_\.\-]?)+)?($authordomain)/", $results))
							{
								
								$p['output']['from']['value'] = '<img src="plugins/dkimstatus/images/authorsign.png" alt="'.$this->gettext('verifiedsender').'" title="'.$this->gettext('verifiedsender').$results.'" /> ' . $p['output']['from']['value'];
							}

							elseif(preg_match("/header\.from=(([a-zA-Z0-9]+[_\.\-]?)+)?($authordomain)/", $results))
							{
								
								$p['output']['from']['value'] = '<img src="plugins/dkimstatus/images/authorsign.png" alt="'.$this->gettext('verifiedsender').'" title="'.$this->gettext('verifiedsender').$results.'" /> ' . $p['output']['from']['value'];
							}
							else {
								$p['output']['from']['value'] = '<img src="plugins/dkimstatus/images/thirdpty.png" alt="'.$this->gettext('thirdpartysig').'" title="'.$this->gettext('thirdpartysig').$results.'" /> ' . $p['output']['from']['value'];
							}
						}

					}
					/* If signature proves invalid, show appropriate warning
					*/ 
					else {
						$p['output']['from']['value'] = '<img src="plugins/dkimstatus/images/invalidsig.png" alt="'.$this->gettext('invalidsignature').'" title="'.$this->gettext('invalidsignature').$results.'" /> ' . $p['output']['from']['value'];
					}
				}
			}
		}
		else {
			$p['output']['from']['value'] = '<img src="plugins/dkimstatus/images/nosiginfo.png" alt="'.$this->gettext('nosignature').'" title="'.$this->gettext('nosignature').'" /> ' . $p['output']['from']['value'];
		}
		return $p;
	}
} 
